# https://pydantic-docs.helpmanual.io/
import numpy as np
import uuid
from datetime import datetime
from typing import List, Union
from attr import attrib, attrs


def list_of_humans_converter(
        possible_humans: Union[List[dict], List['Human']]) -> List['Human']:
    return [
        Human(**x) if isinstance(x, dict) else x
        for x in possible_humans
        if isinstance(x, (Human, dict))
    ]


@attrs(frozen=True)
class Human(object):
    """
    Represents the type of a value an attribute can hold
    """
    name = attrib(type=str)
    dob = attrib(type=datetime)
    friends = attrib(
        type=List['Human'],
        converter=list_of_humans_converter,
        factory=list
    )


def generate_alt_1(num_to_generate):
    for x in range(num_to_generate):
        yield Human(
            name=str(uuid.uuid4()),
            dob=datetime.utcnow(),
            friends=[
                dict(name=str(uuid.uuid4()), dob=datetime.utcnow())
            ]
        )


def generate_alt_2(num_to_generate):
    return map(
        lambda x: Human(
            name=str(uuid.uuid4()),
            dob=datetime.utcnow(),
            friends=[
                dict(name=str(uuid.uuid4()), dob=datetime.utcnow())
            ]
        ),
        np.ones((num_to_generate, 1))
    )


if __name__ == "__main__":
    jack = Human(name=str(uuid.uuid4()), dob=datetime.utcnow())
    jill = Human(name="jill", dob=datetime.utcnow(), friends=[jack])
    print(jack)
    print(jill)



# https://pydantic-docs.helpmanual.io/
import numpy as np
import uuid
from datetime import datetime


def generate_alt_1(num_to_generate):
    for x in range(num_to_generate):
        yield {
            "name": str(uuid.uuid4()),
            "dob": datetime.utcnow(),
            "friends": [
                {
                    "name": str(uuid.uuid4()),
                    "dob": datetime.utcnow(),
                    "friends": []
                }
            ]
        }


def generate_alt_2(num_to_generate):
    return map(
        lambda x: (
            {
                "name": str(uuid.uuid4()),
                "dob": datetime.utcnow(),
                "friends": [
                    {
                        "name": str(uuid.uuid4()),
                        "dob": datetime.utcnow(),
                        "friends": []
                    }
                ]
            }
        ),
        np.ones((num_to_generate, 1))
    )


if __name__ == "__main__":
    print(next(generate_alt_1(1)))
    print(next(generate_alt_2(1)))



# https://pydantic-docs.helpmanual.io/
import numpy as np
import uuid
from datetime import datetime
from typing import List
from pydantic import BaseModel


class Human(BaseModel):
    name: str
    dob: datetime
    friends: List['Human'] = []


Human.update_forward_refs()


def generate_alt_1(num_to_generate):
    for x in range(num_to_generate):
        yield Human(
            name=str(uuid.uuid4()),
            dob=datetime.utcnow(),
            friends=[
                Human(name=str(uuid.uuid4()), dob=datetime.utcnow())
            ]

        )


def generate_alt_2(num_to_generate):
    return map(
        lambda x: Human(
            name=str(uuid.uuid4()),
            dob=datetime.utcnow(),
            friends=[
                Human(name=str(uuid.uuid4()), dob=datetime.utcnow())
            ]

        ),
        np.ones((num_to_generate, 1))
    )


if __name__ == "__main__":
    jack = Human(name=str(uuid.uuid4()), dob=datetime.utcnow())
    jill = Human(name="jill", dob=datetime.utcnow(), friends=[jack])
    print(jack)
    print(jill)



# https://pydantic-docs.helpmanual.io/
import numpy as np
import uuid
from datetime import datetime
from collections import namedtuple

Human = namedtuple('Human', ["name", "dob", "friends"])


def generate_alt_1(num_to_generate):
    for x in range(num_to_generate):
        yield Human(
            name=str(uuid.uuid4()),
            dob=datetime.utcnow(),
            friends=[
                Human(
                    name=str(uuid.uuid4()),
                    dob=datetime.utcnow(),
                    friends=[]
                )
            ]
        )


def generate_alt_2(num_to_generate):
    return map(
        lambda x: Human(
            name=str(uuid.uuid4()),
            dob=datetime.utcnow(),
            friends=[
                Human(
                    name=str(uuid.uuid4()),
                    dob=datetime.utcnow(),
                    friends=[]
                )
            ]
        ),
        np.ones((num_to_generate, 1))
    )


if __name__ == "__main__":
    print(next(generate_alt_1(1)))
    print(next(generate_alt_2(1)))



# -*- coding: utf8 -*-

from setuptools import setup, find_packages

with open("README.md") as fh:
    long_description = fh.read()

setup(
    # Basic info
    name="demo-py",
    version="0.0.1",
    author='Omar Eid',
    author_email='contact.omar.eid@gmail.com',
    url='https://github.com/mr-uuid/perfpy',
    description='Understanding performance characteristics of common python constructs',
    long_description=long_description,
    long_description_content_type='text/markdown',
    classifiers=[
        'Development Status :: 4 - Beta',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: GNU General Public License (GPL)',
        'Operating System :: POSIX',
        'Programming Language :: Python',
        'Topic :: Software Development :: Libraries',
    ],
    # Packages and depencies
    packages=find_packages(),
    install_requires=[
        "attrs",
        "pydantic",
        "toolz",  # https://toolz.readthedocs.io/en/latest/api.html
        "numpy"
        # 'jinja2',
        # 'invoke>=0.13',
        # 'unidecode',
        # 'six',
    ],
    extras_require={
        'dev': [
            # "nbconvert==5.5.0",
            "ipython",
            "mypy",
            "pytest",
            "pytest-pep8",
            "pytest-profiling",  # https://pypi.org/project/pytest-profiling/
            "pstats-view",  # https://github.com/ssanderson/pstats-view
            "flake8",
            # 'manuel',
            # 'pytest-cov',
            # 'coverage',
            # 'mock',
        ],
    },
    # Data files
    package_data={},
    # Scripts
    entry_points={
        'console_scripts': [
            'python-boilerplate = python_boilerplate.__main__:main'],
    },
    # Other configurations
    zip_safe=False,
    platforms='any'
)
